# Generated by Django 2.2.17 on 2021-06-02 18:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0023_enterpriselearner_enterpriselearnerenrollment'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='amount_learner_paid',
            field=models.FloatField(null=True),
        ),
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='course_list_price',
            field=models.FloatField(null=True),
        ),
    ]



# Generated by Django 3.2.20 on 2023-09-14 09:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0035_auto_20230907_1154'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisesubsidybudget',
            name='subsidy_access_policy_display_name',
            field=models.CharField(max_length=512, null=True),
        ),
    ]



"""
Tests for create_enterprise_learner_enrollment_lpr_v1 management command
"""
from unittest import TestCase, mock

from pytest import mark

from django.core.management import call_command

from enterprise_data.models import EnterpriseLearner, EnterpriseLearnerEnrollment
from enterprise_data.tests.test_utils import EnterpriseLearnerFactory


@mark.django_db
class TestCreateEnterpriseLearnerEnrollmentCommand(TestCase):
    """ Tests for create_enterprise_learner_enrollment_lpr_v1 management command"""

    def setUp(self):
        super().setUp()
        self.uuid = 'a'*32
        self.enterprise_user = EnterpriseLearnerFactory(enterprise_customer_uuid=self.uuid)

    def test_create_enterprise_learner_enrollment_lpr_v1_with_dsc_disabled(self):
        """
        Management command should successfully be able to create EnterpriseLearnerEnrollment with DSC Disabled
        """
        assert EnterpriseLearner.objects.count() == 1
        assert EnterpriseLearnerEnrollment.objects.count() == 0

        args = [self.uuid, self.enterprise_user.enterprise_user_id]
        call_command('create_enterprise_learner_enrollment_lpr_v1', *args)

        assert EnterpriseLearnerEnrollment.objects.count() == 1
        enterprise_learner_enrollment = EnterpriseLearnerEnrollment.objects.filter(
            enterprise_customer_uuid=args[0]
        )
        assert enterprise_learner_enrollment.count() == 1
        assert enterprise_learner_enrollment[0].progress_status is None
        assert enterprise_learner_enrollment[0].letter_grade is None
        assert enterprise_learner_enrollment[0].enterprise_user_id is None
        assert enterprise_learner_enrollment[0].user_username is None
        assert enterprise_learner_enrollment[0].user_email is None
        assert enterprise_learner_enrollment[0].enterprise_user is None

    def test_create_enterprise_learner_enrollment_lpr_v1_with_dsc_enabled(self):
        """
        Management command should successfully be able to create EnterpriseLearnerEnrollment with DSC enabled
        """
        assert EnterpriseLearner.objects.count() == 1
        assert EnterpriseLearnerEnrollment.objects.count() == 0

        args = [self.uuid, self.enterprise_user.enterprise_user_id, '--consent_granted']
        call_command('create_enterprise_learner_enrollment_lpr_v1', *args)

        assert EnterpriseLearnerEnrollment.objects.count() == 1
        enterprise_learner_enrollment = EnterpriseLearnerEnrollment.objects.filter(
            enterprise_customer_uuid=args[0]
        )
        assert enterprise_learner_enrollment.count() == 1
        assert enterprise_learner_enrollment[0].letter_grade is not None
        assert enterprise_learner_enrollment[0].last_activity_date is not None
        assert enterprise_learner_enrollment[0].progress_status is not None
        assert enterprise_learner_enrollment[0].enterprise_user_id is not None
        assert enterprise_learner_enrollment[0].user_username is not None
        assert enterprise_learner_enrollment[0].enterprise_user is not None
        assert enterprise_learner_enrollment[0].user_email is not None
        assert EnterpriseLearner.objects.count() == 1

    def test_create_enterprise_learner_enrollment_lpr_v1_error(self):
        """
        Management command will not create enrollment if an error is thrown
        """
        assert EnterpriseLearnerEnrollment.objects.count() == 0

        args = [self.uuid, self.enterprise_user.enterprise_user_id]
        with mock.patch('enterprise_data.tests.test_utils.EnterpriseLearnerEnrollmentFactory') as mock_factory:
            mock_factory.side_effect = [Exception]
            with self.assertRaises(Exception):
                call_command('create_enterprise_learner_enrollment_lpr_v1', *args)

        assert EnterpriseLearnerEnrollment.objects.count() == 0



"""
Test utils for external link reports.
"""

from collections import OrderedDict
import unittest

from enterprise_reporting.external_resource_link_report import (
    AGGREGATE_REPORT_CSV_HEADER_ROW,
    create_csv_string,
    create_columns_for_aggregate_report,
    process_coursegraph_results,
    split_up_results,
)


class TestUtilsCoursegraph(unittest.TestCase):
    
    def setUp(self):
        """
        Setup reusable test data
        """
        self.raw_data = [
            {'course_title': 'course1',
             'organization': 'edx',
             'h.course_key': 'course-v1:I+am+a+test1',
             'h.data': '<span href="http://www.google.com">my site</span>'},
            {'course_title': 'course1',
             'organization': 'edx',
             'h.course_key': 'course-v1:I+am+a+test1',
             'h.data': '<span href="http://www.google.com">my site is http://www.google.com</span>'},
            {'course_title': 'course1',
             'organization': 'edx',
             'h.course_key': 'course-v1:I+am+a+test1',
             'h.data': '<span href="http://www.facebook.com/">my site</span>'},
            
            {'course_title': 'course2',
             'organization': 'edx',
             'h.course_key': 'course-v1:I+am+a+test2',
             'h.data': '<span href="http://www.google2.com/">my site</span>'},
            {'course_title': 'course2',
             'organization': 'edx',
             'h.course_key': 'course-v1:I+am+a+test2',
             'h.data': '<span href="http://www.google2.com">my site</span>'},
            {'course_title': 'course2',
             'organization': 'edx',
             'h.course_key': 'course-v1:I+am+a+test2',
             'h.data': '<span href="http://www.google2.com/someextension/">my site</span>'},
            {'course_title': 'course2',
             'organization': 'edx',
             'h.course_key': 'course-v1:I+am+a+test2',
             'h.data': '<span href="http://www.google2.com/someextension/">my site</span>'},
            
            {'course_title': 'course3',
             'organization': 'edx2',
             'h.course_key': 'course-v1:I+am+a+test3',
             'h.data': '<span href="http://www.google3.com">my site</span>'},
    
            {'course_title': 'course4',
             'organization': 'edx2',
             'h.course_key': 'course-v1:I+am+a+test4',
             'h.data': 'I have no urls'},
            {'course_title': 'course4',
             'organization': 'edx2',
             'h.course_key': 'course-v1:I+am+a+test4+with+bad+url',
             'h.data': '<p>code choking on https://[Web App Name].scm.azureWeb Apps.net</p>'},

            {'course_title': 'oldcourse',
             'organization': 'oldx2',
             'h.course_key': 'oldx2course:I+am+a+test+for+old+coursekeys',
             'h.data': '<span href="http://www.google3.com">my site</span>'},

        ]

    def test_process_coursegraph_results(self):
        """
        process_results should properly structure the data in a dictionary returned
        """
        expected = {
            'course-v1:I+am+a+test1': {
                'course_title': 'course1',
                'organization': 'edx',
                'external_links': {
                    'http://www.google.com',
                    'http://www.facebook.com/'
                },
                'domain_count': {
                    'http://www.google.com': 3,
                    'http://www.facebook.com': 1,
                },
            },
            'course-v1:I+am+a+test2': {
                'course_title': 'course2',
                'organization': 'edx',
                'external_links': {
                    'http://www.google2.com/',
                    'http://www.google2.com',
                    'http://www.google2.com/someextension/',
                },
                'domain_count': {
                    'http://www.google2.com': 4,
                },
            },
            'course-v1:I+am+a+test3': {
                'course_title': 'course3',
                'organization': 'edx2',
                'external_links': {'http://www.google3.com'},
                'domain_count': {
                    'http://www.google3.com': 1,
                },
            },
        }
        assert process_coursegraph_results(self.raw_data) == expected

    def test_split_up_results(self):
        """
        split_up_results should take a dictionary of process results and split
        it into a list of smaller dictionaries.
        """
        processed_results = OrderedDict([
            ('course-v1:I+am+a+test1', {
                'course_title': 'course1',
                'organization': 'edx',
                'domain_count': OrderedDict([
                    ('http://www.google.com', 2),
                    ('http://www.facebook.com', 1),
                ]),
            }),
            ('course-v1:I+am+a+test2', {
                'course_title': 'course2',
                'organization': 'edx',
                'domain_count': {'http://www.google2.com': 4},
            }),
            ('course-v1:I+am+a+test3', {
                'course_title': 'course3',
                'organization': 'edx2',
                'domain_count': {'http://www.google3.com': 1},
            }),
        ])

        results_list = split_up_results(processed_results)

        assert len(results_list) == 2
        assert len(results_list[0]) == 1
        assert len(results_list[1]) == 2

    def test_create_aggregate_report_csv_string(self):
        """
        generate_aggregate_report_csv_string should create the expected csv
        string given some processed results
        """

        processed_results = OrderedDict([
            ('course-v1:I+am+a+test1', {
                'course_title': 'course1',
                'organization': 'edx',
                'domain_count': OrderedDict([
                    ('http://www.google.com', 2),
                    ('http://www.facebook.com', 1),
                ]),
            }),
            ('course-v1:I+am+a+test2', {
                'course_title': 'course2',
                'organization': 'edx',
                'domain_count': {'http://www.google2.com': 4},
            }),
            ('course-v1:I+am+a+test3', {
                'course_title': 'course3',
                'organization': 'edx2',
                'domain_count': {'http://www.google3.com': 1},
            }),
        ])
        expected = (
            'Course Key,Course Title,Partner,External Domain,Count\n'
            'course-v1:I+am+a+test1,"course1",edx,http://www.google.com,2\n'
            ',,,http://www.facebook.com,1\n'
            'course-v1:I+am+a+test2,"course2",edx,http://www.google2.com,4\n'
            'course-v1:I+am+a+test3,"course3",edx2,http://www.google3.com,1\n'
        )
        actual = create_csv_string(
            processed_results,
            AGGREGATE_REPORT_CSV_HEADER_ROW,
            create_columns_for_aggregate_report,
        )
        assert actual == expected



"""
Database models for enterprise data.
"""


from logging import getLogger

from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

LOGGER = getLogger(__name__)


class EnterpriseReportingModelManager(models.Manager):
    """
    Custom ModelManager for every model that wants to use `enterprise_reporting` database.
    """

    def get_queryset(self):
        """
        Override to use the `enterprise_reporting` database instead of default.
        """
        qs = super().get_queryset()
        qs = qs.using(settings.ENTERPRISE_REPORTING_DB_ALIAS)

        return qs


class EnterpriseLearner(models.Model):
    """
    Information related to Enterprise Learner.
    """

    objects = EnterpriseReportingModelManager()

    class Meta:
        app_label = 'enterprise_data'
        db_table = 'enterprise_learner'
        verbose_name = _("Enterprise Learner")
        verbose_name_plural = _("Enterprise Learner")
        ordering = ['-user_email']

    enterprise_user_id = models.PositiveIntegerField(primary_key=True)
    enterprise_customer_uuid = models.UUIDField(db_index=True, null=False)
    enterprise_user_created = models.DateTimeField(null=True)
    enterprise_user_modified = models.DateTimeField(null=True)
    enterprise_user_active = models.BooleanField(default=False)
    lms_user_id = models.PositiveIntegerField()
    is_linked = models.BooleanField(default=False)
    user_username = models.CharField(max_length=255, null=True)
    user_email = models.CharField(max_length=255, null=True, db_index=True)
    lms_user_created = models.DateTimeField(null=True)
    lms_last_login = models.DateTimeField(null=True)
    lms_user_country = models.CharField(max_length=2, null=True)
    enterprise_sso_uid = models.CharField(max_length=255, null=True)
    last_activity_date = models.DateField(null=True, db_index=True)
    created_at = models.DateTimeField(null=True, db_index=True)


class EnterpriseLearnerEnrollment(models.Model):
    """
    Information related to Enterprise Learner Enrollments.
    """

    objects = EnterpriseReportingModelManager()

    class Meta:
        app_label = 'enterprise_data'
        db_table = 'enterprise_learner_enrollment'
        verbose_name = _("Enterprise Learner Enrollment")
        verbose_name_plural = _("Enterprise Learner Enrollments")

    enterprise_enrollment_id = models.PositiveIntegerField(primary_key=True)
    enrollment_id = models.PositiveIntegerField(null=True)
    is_consent_granted = models.BooleanField(default=False)
    paid_by = models.CharField(max_length=255, null=True)
    user_current_enrollment_mode = models.CharField(max_length=32)
    enrollment_date = models.DateField()
    unenrollment_date = models.DateField(null=True)
    unenrollment_end_within_date = models.DateField(null=True)
    is_refunded = models.BooleanField(default=None, null=True)
    seat_delivery_method = models.CharField(max_length=255, null=True)
    offer_id = models.CharField(max_length=255, null=True)
    offer_name = models.CharField(max_length=255, null=True)
    offer_type = models.CharField(max_length=255, null=True)
    coupon_code = models.CharField(max_length=255, null=True)
    coupon_name = models.CharField(max_length=255, null=True)
    contract_id = models.CharField(max_length=255, null=True)
    course_list_price = models.FloatField(null=True)
    amount_learner_paid = models.FloatField(null=True)
    course_key = models.CharField(max_length=255, null=False, db_index=True)
    courserun_key = models.CharField(max_length=255, null=False, db_index=True)
    course_title = models.CharField(max_length=255, null=True, db_index=True)
    course_pacing_type = models.CharField(max_length=32, null=True)
    course_start_date = models.DateField(null=True, db_index=True)
    course_end_date = models.DateField(null=True)
    course_duration_weeks = models.PositiveIntegerField(null=True)
    course_max_effort = models.PositiveIntegerField(null=True)
    course_min_effort = models.PositiveIntegerField(null=True)
    course_primary_program = models.CharField(max_length=255, null=True)
    primary_program_type = models.CharField(max_length=255, null=True)
    course_primary_subject = models.CharField(max_length=255, null=True)
    has_passed = models.BooleanField(default=False)
    last_activity_date = models.DateField(null=True, db_index=True)
    total_learning_time_seconds = models.PositiveIntegerField(default=0)
    progress_status = models.CharField(max_length=255, null=True)
    passed_date = models.DateField(null=True)
    current_grade = models.FloatField(null=True)
    letter_grade = models.CharField(max_length=64, null=True)
    enterprise_user = models.ForeignKey(
        'EnterpriseLearner',
        related_name='enrollments',
        to_field='enterprise_user_id',
        on_delete=models.CASCADE,
        null=True,
    )
    user_email = models.CharField(max_length=255, null=True, db_index=True)
    user_account_creation_date = models.DateTimeField(null=True)
    user_country_code = models.CharField(max_length=2, null=True)
    user_username = models.CharField(max_length=255, null=True)
    enterprise_name = models.CharField(max_length=255, db_index=True, null=False)
    enterprise_customer_uuid = models.UUIDField(db_index=True, null=False)
    enterprise_sso_uid = models.CharField(max_length=255, null=True)
    created = models.DateTimeField(null=True, db_index=True)
    is_subsidy = models.BooleanField(default=False)
    course_product_line = models.CharField(max_length=64, null=True)
    budget_id = models.UUIDField(db_index=True, null=True)


class EnterpriseAdminLearnerProgress(models.Model):
    """
    Aggregated data on learners at different risk points, either during subsidy activation or throughout the course.
    """

    objects = EnterpriseReportingModelManager()

    class Meta:
        app_label = 'enterprise_data'
        db_table = 'enterprise_admin_learner_progress'
        verbose_name = _("Enterprise Admin Learner Progress")
        verbose_name_plural = _("Enterprise Admin Learner Progress")

    enterprise_customer_uuid = models.UUIDField(primary_key=True)
    enterprise_customer_name = models.CharField(max_length=255)
    active_subscription_plan = models.BooleanField(
        help_text='Whether the subscription plan is active or nor?'
    )
    assigned_licenses = models.PositiveIntegerField(
        help_text='Number of assigned licenses'
    )
    activated_licenses = models.PositiveIntegerField(
        help_text='Number of active licenses'
    )
    assigned_licenses_percentage = models.FloatField(
        help_text='Float representation of percentage of assigned licenses out of total'
    )
    activated_licenses_percentage = models.FloatField(
        help_text='Float representation of percentage of active licenses out of total'
    )
    active_enrollments = models.PositiveIntegerField(
        help_text='Number of enrollments in active courses'
    )
    at_risk_enrollment_less_than_one_hour = models.PositiveIntegerField(
        help_text='Number of enrollments from the past 30 days that have not yet achieved 1 hour of learning time'
    )
    at_risk_enrollment_end_date_soon = models.PositiveIntegerField(
        help_text=(
            'Number of enrollments have end dates coming up in the next 30 days. Course complete time is almost up'
        )
    )
    at_risk_enrollment_dormant = models.PositiveIntegerField(
        help_text='Number of dormant enrollments in the past 30 days'
    )
    created_at = models.DateTimeField()


class EnterpriseAdminSummarizeInsights(models.Model):
    """
    Aggregated data on enrollments, daily sessions, learning hours, and completions for the past 60 days.
    """

    objects = EnterpriseReportingModelManager()

    class Meta:
        app_label = 'enterprise_data'
        db_table = 'enterprise_admin_summarize_insights'
        verbose_name = _("Enterprise Admin Summarize Insights")
        verbose_name_plural = _("Enterprise Admin Summarize Insights")

    enterprise_customer_uuid = models.UUIDField(primary_key=True)
    enterprise_customer_name = models.CharField(max_length=255)
    enrolls = models.PositiveIntegerField(help_text='Total number of enrollments in last 30 days')
    enrolls_prior = models.PositiveIntegerField(help_text='Total number of enrollments in last 31-60 days')
    passed = models.PositiveIntegerField(help_text='Total number of completions in last 30 days')
    passed_prior = models.PositiveIntegerField(help_text='Total number of completions in last 31-60 days')
    engage = models.PositiveIntegerField(help_text='Total number of daily sessions/engagements in last 30 days')
    engage_prior = models.PositiveIntegerField(
        help_text='Total number of daily sessions/engagements in last 31-60 days'
    )
    hours = models.PositiveIntegerField(help_text='Total number of learning hours in last 30 days')
    hours_prior = models.PositiveIntegerField(help_text='Total number of learning hours in last 31-60 days')
    contract_end_date = models.DateTimeField(help_text='Contract end date')
    active_contract = models.BooleanField(help_text='Whether or not the customer has a contract?')
    created_at = models.DateTimeField()


class EnterpriseEnrollment(models.Model):
    """Enterprise Enrollment is the learner details for a specific course enrollment.

    This information includes a mix of the learners meta data (email, username, etc), the enterprise they are
    associated with (enterprise_id, enterprise_name, etc), and their status in the course (course_id,
    letter_grade, has_passed, etc).
    This is meant to closely mirror the data the Enterprises recieve in the automated report (see enterprise_reporting
    folder in this repo), with the exception of some data that is only calculated in Vertica as this data is pulled from
    the analytics result store via the analytics-data-api project.
    """

    class Meta:
        app_label = 'enterprise_data'
        db_table = 'enterprise_enrollment'
        verbose_name = _("Enterprise Enrollment")
        verbose_name_plural = _("Enterprise Enrollments")

    enterprise_id = models.UUIDField()
    enterprise_name = models.CharField(max_length=255)
    lms_user_id = models.PositiveIntegerField()
    enterprise_user = models.ForeignKey(
        'EnterpriseUser',
        related_name='enrollments',
        to_field='enterprise_user_id',
        on_delete=models.CASCADE,
    )
    course_id = models.CharField(max_length=255, help_text='The course the learner is enrolled in.')
    enrollment_created_timestamp = models.DateTimeField()
    user_current_enrollment_mode = models.CharField(max_length=32)
    consent_granted = models.BooleanField(default=None, null=True)
    letter_grade = models.CharField(max_length=32, null=True)
    has_passed = models.BooleanField(default=False)
    passed_timestamp = models.DateTimeField(null=True)
    enterprise_sso_uid = models.CharField(max_length=255, null=True)
    enterprise_site_id = models.PositiveIntegerField(null=True)
    course_title = models.CharField(max_length=255, null=True)
    course_start = models.DateTimeField(null=True)
    course_end = models.DateTimeField(null=True)
    course_pacing_type = models.CharField(max_length=32, null=True)
    course_duration_weeks = models.CharField(max_length=32, null=True)
    course_min_effort = models.PositiveIntegerField(null=True)
    course_max_effort = models.PositiveIntegerField(null=True)
    user_account_creation_timestamp = models.DateTimeField(null=True)
    user_email = models.CharField(max_length=255, null=True)
    user_username = models.CharField(max_length=255, null=True)
    course_key = models.CharField(max_length=255, null=True)
    user_country_code = models.CharField(max_length=2, null=True)
    last_activity_date = models.DateField(null=True)
    coupon_name = models.CharField(max_length=128, null=True)
    coupon_code = models.CharField(max_length=128, null=True)
    offer = models.CharField(max_length=128, null=True)
    current_grade = models.FloatField(null=True)
    course_price = models.DecimalField(decimal_places=2, max_digits=12, null=True)
    discount_price = models.DecimalField(decimal_places=2, max_digits=12, null=True)
    created = models.DateTimeField(null=True)
    unenrollment_timestamp = models.DateTimeField(null=True)

    def __str__(self):
        """
        Return a human-readable string representation of the object.
        """
        return "<Enterprise Enrollment for user {user} in {course}>".format(
            user=self.enterprise_user_id,
            course=self.course_id
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()


class EnterpriseOffer(models.Model):
    """
    Details for an enterprise offer.

    """

    objects = EnterpriseReportingModelManager()

    class Meta:
        app_label = 'enterprise_data'
        db_table = 'enterprise_offer_aggregates'
        verbose_name = _("Enterprise Offer")
        verbose_name_plural = _("Enterprise Offers")

    offer_id = models.CharField(max_length=255, primary_key=True)
    enterprise_customer_uuid = models.UUIDField()
    enterprise_name = models.CharField(max_length=255, null=True)
    sum_amount_learner_paid = models.FloatField(null=True)
    sum_course_price = models.FloatField(null=True)
    status = models.CharField(max_length=255, null=True)
    offer_type = models.CharField(max_length=255, null=True)
    date_created = models.DateTimeField(null=True)
    emails_for_usage_alert = models.CharField(max_length=1023, null=True)
    discount_type = models.CharField(max_length=64, null=True)
    discount_value = models.FloatField(null=True)
    max_discount = models.FloatField(null=True)
    total_discount_ecommerce = models.FloatField(null=True)
    amount_of_offer_spent = models.FloatField(null=True)
    percent_of_offer_spent = models.FloatField(null=True)
    remaining_balance = models.FloatField(null=True)
    amount_offer_spent_ocm = models.FloatField(null=True)
    amount_offer_spent_exec_ed = models.FloatField(null=True)
    export_timestamp = models.DateTimeField(null=True)

    def __str__(self):
        """
        Return a human-readable string representation of the object.
        """
        return f'<Enterprise Offer {self.offer_id} for {self.enterprise_name}>'

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()


class EnterpriseSubsidyBudget(models.Model):
    """
    Details for budgets in an enterprise offer.

    """

    objects = EnterpriseReportingModelManager()

    class Meta:
        app_label = 'enterprise_data'
        db_table = 'ent_subsidy_access_policy_aggregates'
        verbose_name = _("Enterprise Subsidy Budget")
        verbose_name_plural = _("Enterprise Subsidy Budgets")

    id = models.CharField(
        db_index=True,
        primary_key=True,
        max_length=32,
        help_text="Hashed surrogate key based on subsidy_access_policy_uuid and subsidy_uuid"
    )
    subsidy_access_policy_uuid = models.UUIDField(help_text="Budget Id")
    subsidy_uuid = models.UUIDField()
    enterprise_customer_uuid = models.UUIDField()
    enterprise_customer_name = models.CharField(max_length=255, null=True)
    subsidy_access_policy_description = models.CharField(max_length=500, null=True)
    subsidy_access_policy_display_name = models.CharField(max_length=512, null=True)
    subsidy_title = models.CharField(max_length=255, null=True)
    catalog_name = models.CharField(max_length=255, null=True)
    subsidy_access_policy_type = models.CharField(max_length=255, null=True)
    date_created = models.DateTimeField(null=True)
    subsidy_start_datetime = models.DateTimeField(null=True)
    subsidy_expiration_datetime = models.DateTimeField(null=True)
    is_active = models.BooleanField(default=False)
    is_test = models.BooleanField(default=False)
    ocm_usage = models.FloatField(null=True)
    exec_ed_usage = models.FloatField(null=True)
    usage_total = models.FloatField(null=True)
    enterprise_contract_discount_percent = models.FloatField(null=True)
    starting_balance = models.FloatField(null=True)
    amount_of_policy_spent = models.FloatField(null=True)
    remaining_balance = models.FloatField(null=True)
    percent_of_policy_spent = models.FloatField(null=True)

    def __str__(self):
        """
        Return a human-readable string representation of the object.
        """
        return (f'<Enterprise Budget {self.subsidy_access_policy_uuid} for '
                f'Subsidy {self.subsidy_uuid} for {self.enterprise_customer_name}>')

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()


class EnterpriseUser(models.Model):
    """Information includes a mix of the user's meta data.

    Includes (lms_user_id, username, etc), and the enterprise they are associated with (enterprise_id).
    """

    class Meta:
        app_label = 'enterprise_data'
        db_table = 'enterprise_user'
        verbose_name = _("Enterprise User")
        verbose_name_plural = _("Enterprise Users")
        ordering = ['-user_email']

    enterprise_id = models.UUIDField()
    lms_user_id = models.PositiveIntegerField()
    enterprise_user_id = models.PositiveIntegerField(unique=True)
    enterprise_sso_uid = models.CharField(max_length=255, null=True)
    user_account_creation_timestamp = models.DateTimeField(null=True)
    user_email = models.CharField(max_length=255, null=True)
    user_username = models.CharField(max_length=255, null=True)
    user_country_code = models.CharField(max_length=2, null=True)
    last_activity_date = models.DateField(null=True)
    created = models.DateTimeField(null=True)

    def __str__(self):
        """
        Return a human-readable string representation of the object.
        """
        return "<Enterprise User {user} in {enterprise}>".format(
            user=self.lms_user_id,
            enterprise=self.enterprise_id
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()



"""
Tests for create_enterprise_learner_lpr_v1 management command
"""
from unittest import TestCase, mock

from pytest import mark

from django.core.management import call_command

from enterprise_data.models import EnterpriseLearner
from enterprise_data.tests.test_utils import EnterpriseLearnerFactory


@mark.django_db
class TestCreateEnterpriseLearnerCommand(TestCase):
    """
    Tests to validate the behavior of `./manage.py create_enterprise_learner_lpr_v1` management command.
    """

    def setUp(self):
        super().setUp()
        self.uuid = 'a'*32

    def test_create_enterprise_learner(self):
        """
        Management command should successfully be able to create EnterpriseLearner
        """
        assert EnterpriseLearner.objects.count() == 0

        args = [self.uuid]
        call_command('create_enterprise_learner_lpr_v1', *args)

        assert EnterpriseLearner.objects.count() == 1
        assert EnterpriseLearner.objects.filter(enterprise_customer_uuid=args[0]).count() == 1

    def test_create_enterprise_learner_error(self):
        """
        Management command should handle exceptions gracefully.
        """
        EnterpriseLearnerFactory(enterprise_customer_uuid=self.uuid)
        assert EnterpriseLearner.objects.count() == 1

        args = [self.uuid]
        with mock.patch('enterprise_data.tests.test_utils.EnterpriseLearnerFactory') as mock_factory:
            mock_factory.side_effect = [Exception]
            with self.assertRaises(Exception):
                call_command('create_enterprise_learner_lpr_v1', *args)

        assert EnterpriseLearner.objects.count() == 1



# Generated by Django 3.2.12 on 2022-06-09 10:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0026_auto_20210916_0414'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterpriselearnerenrollment',
            name='total_learning_time_seconds',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



# Generated by Django 3.2.19 on 2023-08-02 09:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0032_auto_20230704_0818'),
    ]

    operations = [
        migrations.CreateModel(
            name='EnterpriseAdminLearnerProgress',
            fields=[
                ('enterprise_customer_uuid', models.UUIDField(primary_key=True, serialize=False)),
                ('enterprise_customer_name', models.CharField(max_length=255)),
                ('active_subscription_plan', models.BooleanField(help_text='Whether the subscription plan is active or nor?')),
                ('assigned_licenses', models.PositiveIntegerField(help_text='Number of assigned licenses')),
                ('activated_licenses', models.PositiveIntegerField(help_text='Number of active licenses')),
                ('assigned_licenses_percentage', models.FloatField(help_text='Float representation of percentage of assigned licenses out of total')),
                ('activated_licenses_percentage', models.FloatField(help_text='Float representation of percentage of active licenses out of total')),
                ('active_enrollments', models.PositiveIntegerField(help_text='Number of enrollments in active courses')),
                ('at_risk_enrollment_less_than_one_hour', models.PositiveIntegerField(help_text='Number of enrollments from the past 30 days that have not yet achieved 1 hour of learning time')),
                ('at_risk_enrollment_end_date_soon', models.PositiveIntegerField(help_text='Number of enrollments have end dates coming up in the next 30 days. Course complete time is almost up')),
                ('at_risk_enrollment_dormant', models.PositiveIntegerField(help_text='Number of dormant enrollments in the past 30 days')),
                ('created_at', models.DateTimeField()),
            ],
            options={
                'verbose_name': 'Enterprise Admin Learner Progress',
                'verbose_name_plural': 'Enterprise Admin Learner Progress',
                'db_table': 'enterprise_admin_learner_progress',
            },
        ),
        migrations.CreateModel(
            name='EnterpriseAdminSummarizeInsights',
            fields=[
                ('enterprise_customer_uuid', models.UUIDField(primary_key=True, serialize=False)),
                ('enterprise_customer_name', models.CharField(max_length=255)),
                ('enrolls', models.PositiveIntegerField(help_text='Total number of enrollments in last 30 days')),
                ('enrolls_prior', models.PositiveIntegerField(help_text='Total number of enrollments in last 31-60 days')),
                ('passed', models.PositiveIntegerField(help_text='Total number of completions in last 30 days')),
                ('passed_prior', models.PositiveIntegerField(help_text='Total number of completions in last 31-60 days')),
                ('engage', models.PositiveIntegerField(help_text='Total number of daily sessions/engagements in last 30 days')),
                ('engage_prior', models.PositiveIntegerField(help_text='Total number of daily sessions/engagements in last 31-60 days')),
                ('hours', models.PositiveIntegerField(help_text='Total number of learning hours in last 30 days')),
                ('hours_prior', models.PositiveIntegerField(help_text='Total number of learning hours in last 31-60 days')),
                ('contract_end_date', models.DateTimeField(help_text='Contract end date')),
                ('active_contract', models.BooleanField(help_text='Whether or not the customer has a contract?')),
                ('created_at', models.DateTimeField()),
            ],
            options={
                'verbose_name': 'Enterprise Admin Summarize Insights',
                'verbose_name_plural': 'Enterprise Admin Summarize Insights',
                'db_table': 'enterprise_admin_summarize_insights',
            },
        ),
    ]



"""
Test send enterprise reports.
"""

import datetime
import unittest
from collections import namedtuple

import pytz

from enterprise_reporting.send_enterprise_reports import should_deliver_report
from enterprise_reporting.utils import FREQUENCY_TYPE_DAILY


class TestSendEnterpriseReports(unittest.TestCase):

	def test_should_deliver_report(self):
		"""
		Verify that `should_deliver_report` works.
		"""
		est_timezone = pytz.timezone('US/Eastern')
		current_est_time = datetime.datetime.now(est_timezone)
		reporting_config = {
			'active': True,
			'data_type': 'progress_v3',
			'frequency': FREQUENCY_TYPE_DAILY,
			'hour_of_day': current_est_time.hour,
			'day_of_month': current_est_time.day,
			'day_of_week': current_est_time.weekday(),
		}
		# these are passed from jenkins when we manually execute an enterprise report job
		# in case of automated jobs, these are empty
		Command = namedtuple("Command", "data_type enterprise_customer")
		args = Command('', '')

		assert should_deliver_report(args, reporting_config)



"""
URL definitions for enterprise data api v1.
"""


from rest_framework.routers import DefaultRouter

from django.urls import re_path

from enterprise_data.api.v1 import views
from enterprise_data.constants import UUID4_REGEX

app_name = 'enterprise_data_api_v1'

router = DefaultRouter()
router.register(
    r'enterprise/(?P<enterprise_id>.+)/enrollments',
    views.EnterpriseLearnerEnrollmentViewSet,
    'enterprise-learner-enrollment',
)
router.register(
    r'enterprise/(?P<enterprise_id>.+)/offers',
    views.EnterpriseOfferViewSet,
    'enterprise-offers',
)
router.register(
    r'enterprise/(?P<enterprise_id>.+)/users',
    views.EnterpriseLearnerViewSet,
    'enterprise-learner',
)
router.register(
    r'enterprise/(?P<enterprise_id>.+)/learner_completed_courses',
    views.EnterpriseLearnerCompletedCoursesViewSet,
    'enterprise-learner-completed-courses',
)

urlpatterns = [
    re_path(
        fr'^admin/insights/(?P<enterprise_id>{UUID4_REGEX})$',
        views.EnterpriseAdminInsightsView.as_view(),
        name='enterprise-admin-insights'
    ),
]

urlpatterns += router.urls
